<?php

namespace EventPortal\Tests\Unit\Event\Models;

use Carbon\Carbon;
use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Agenda\Models\Agenda;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Team\Models\Team;
use EventPortal\Tests\Support\Traits\InteractsWithTeams;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class EventTest extends TestCase
{
    use RefreshDatabase, InteractsWithTeams, SeedRolesAndPermissions;

    /** @test **/
    public function event_is_audited_when_updated()
    {
        config(['audit.console' => true]); // This needs to be before create, or no update audit is created

        $event = factory(Event::class)->create(['name' => 'Hello World!']);
        $event->update(['name' => 'Updated World!']);

        $this->assertCount(2, $event->audits); // Created and updated
        $updatedAudit = $event->fresh()->audits->last();

        $this->assertArraySubset([
            'name' => 'Hello World!',
        ], $updatedAudit->old_values);
        $this->assertArraySubset([
            'name' => 'Updated World!',
        ], $updatedAudit->new_values);
    }

    /** @test */
    public function filters_out_events_for_admins_who_are_in_a_team(): void
    {
        $team = factory(Team::class)->create();
        $adminTeam = factory(Team::class)->create();
        $admin = factory(Admin::class)->state('admin')->create();
        $event = factory(Event::class)->create();
        $adminEvent = factory(Event::class)->create();

        $this->addToSameTeam([$admin, $adminEvent], $adminTeam);
        $this->addToSameTeam([$event], $team);

        $this->actingAsAdmin($admin);
        $result = Event::forAdminInTeam()->get();

        $this->assertCount(1, $result);
    }

    /** @test */
    public function does_not_filter_out_events_for_super_admins_who_are_in_a_team(): void
    {
        $team = factory(Team::class)->create();
        $superAdminTeam = factory(Team::class)->create();
        $superAdmin = factory(Admin::class)->state('super-admin')->create();
        $event = factory(Event::class)->create();
        $superAdminEvent = factory(Event::class)->create();

        $this->addToSameTeam([$superAdmin, $superAdminEvent], $superAdminTeam);
        $this->addToSameTeam([$event], $team);

        $this->actingAsSuperAdmin($superAdmin);
        $result = Event::forAdminInTeam()->get();

        $this->assertCount(2, $result);
    }

    /** @test **/
    public function can_get_single_readable_date()
    {
        $agenda = factory(Agenda::class)->create(['date' => '2020-07-31']);

        $this->assertEquals('31/07/2020', $agenda->event->readable_dates);
    }

    /** @test **/
    public function can_get_range_readable_date()
    {
        $event = factory(Event::class)->create();

        factory(Agenda::class)->create([
            'event_id' => $event->id,
            'date' => '2020-07-31',
        ]);

        factory(Agenda::class)->create([
            'event_id' => $event->id,
            'date' => '2020-08-02',
        ]);

        factory(Agenda::class)->create([
            'event_id' => $event->id,
            'date' => '2020-08-04',
        ]);

        $this->assertEquals('31/07/2020 - 04/08/2020', $event->readable_dates);
    }

    /** @test **/
    public function can_get_range_readable_date_based_on_date()
    {
        $event = factory(Event::class)->create();

        factory(Agenda::class)->create([
            'event_id' => $event->id,
            'date' => '2020-07-31',
        ]);

        factory(Agenda::class)->create([
            'event_id' => $event->id,
            'date' => '2020-09-04',
        ]);

        factory(Agenda::class)->create([
            'event_id' => $event->id,
            'date' => '2020-08-02',
        ]);

        $this->assertEquals('31/07/2020 - 04/09/2020', $event->readable_dates);
    }

    /** @test **/
    public function has_events_before_or_equal_to()
    {
        $event = factory(Event::class)->create();

        factory(Agenda::class)->create([
            'event_id' => $event->id,
            'date' => '2020-07-31',
        ]);

        $this->assertNotContainsModel($event, Event::hasDateBeforeOrEqualTo('30/07/2020')->get());
        $this->assertContainsModel($event, Event::hasDateBeforeOrEqualTo('01/08/2020')->get());
    }

    /** @test **/
    public function has_events_after_or_equal_to()
    {
        $event = factory(Event::class)->create();

        factory(Agenda::class)->create([
            'event_id' => $event->id,
            'date' => '2020-07-31',
        ]);

        $this->assertContainsModel($event, Event::hasDateAfterOrEqualTo('30/07/2020')->get());
        $this->assertNotContainsModel($event, Event::hasDateAfterOrEqualTo('01/08/2020')->get());
    }

    /** @test **/
    public function has_events_before_or_equal_to_middle_of_range()
    {
        $event = factory(Event::class)->create();

        factory(Agenda::class)->create([
            'event_id' => $event->id,
            'date' => '2020-07-01',
        ]);

        factory(Agenda::class)->create([
            'event_id' => $event->id,
            'date' => '2020-07-31',
        ]);

        $this->assertContainsModel($event, Event::hasDateBeforeOrEqualTo('15/07/2020')->get());
    }

    /** @test **/
    public function has_events_after_or_equal_to_middle_of_range()
    {
        $event = factory(Event::class)->create();

        factory(Agenda::class)->create([
            'event_id' => $event->id,
            'date' => '2020-07-01',
        ]);

        factory(Agenda::class)->create([
            'event_id' => $event->id,
            'date' => '2020-07-31',
        ]);

        $this->assertContainsModel($event, Event::hasDateAfterOrEqualTo('15/07/2020')->get());
    }

    /** @test */
    public function viewable_scope_test()
    {
        factory(Event::class, 5)->state('draft')->create();
        factory(Event::class, 5)->state('published')->create();
        factory(Event::class, 5)->state('post')->create();
        factory(Event::class, 5)->state('cancelled')->create();
        factory(Event::class, 5)->state('archived')->create();

        $this->assertCount(25, Event::all('id'));
        $this->assertCount(15, Event::viewable()->get('id'));
    }

    /** @test */
    public function event_can_have_snippet_and_description()
    {
        $event = factory(Event::class)->create();
        $this->assertNull($event->snippet);
        $this->assertNull($event->description);

        $event->update([
            'snippet' => 'wow, such snippet',
            'description' => 'an description',
        ]);

        $this->assertNotNull($event->snippet);
        $this->assertNotNull($event->description);
        $this->assertEquals('wow, such snippet', $event->snippet);
        $this->assertEquals('an description', $event->description);
    }

    /** @test */
    public function get_upcoming_events_for_delegate()
    {
        $upcoming = factory(Event::class)->create();
        $upcoming->agendas()->create([
            'date' => now()->addDay(),
        ]);

        $previous = factory(Event::class)->create();
        $previous->agendas()->create([
            'date' => now()->subDays(),
        ]);

        $this->assertCount(1, Event::upcoming()->get());
        $this->assertSameModel($upcoming, Event::upcoming()->first());
    }

    /** @test */
    public function get_previous_events_for_delegate()
    {
        $previous = factory(Event::class)->create();
        $previous->agendas()->create([
            'date' => now()->subDays(),
        ]);

        $upcoming = factory(Event::class)->create();
        $upcoming->agendas()->create([
            'date' => now()->addDay(),
        ]);

        $this->assertCount(1, Event::previous()->get());
        $this->assertSameModel($previous, Event::previous()->first());
    }

    /** @test */
    public function get_start_date()
    {
        $event = factory(Event::class)->create();

        $event->agendas()->create([
            'date' => Carbon::parse('09 September 2020'),
        ]);

        $event->agendas()->create([
            'date' => Carbon::parse('08 September 2020'),
        ]);

        $this->assertEquals(Carbon::parse('08 September 2020'), $event->startDate);
    }

    /** @test */
    public function handle_no_start_date()
    {
        $event = factory(Event::class)->create();

        $this->assertEquals(null, $event->startDate);
    }

    /** @test */
    public function get_readable_start_date()
    {
        $event = factory(Event::class)->create();

        $event->agendas()->create([
            'date' => Carbon::parse('09th September 2020'),
        ]);

        $event->agendas()->create([
            'date' => Carbon::parse('08th September 2020'),
        ]);

        $this->assertEquals('08th September 2020', $event->readableStartDate);
    }

    /** @test */
    public function handle_no_readable_start_date()
    {
        $event = factory(Event::class)->create();

        $this->assertEquals(null, $event->readableStartDate);
    }

    /** @test */
    public function get_event_for_delegate()
    {
        $event = factory(Event::class)->create();
        $delegate = factory(Delegate::class)->create();
        $event->delegates()->save($delegate, ['status' => 'draft', 'pathway_id' => 1]);

        $eventFromDelegate = Event::forDelegate($delegate)->first();

        $this->assertSameModel($event, $eventFromDelegate);
    }

    /** @test **/
    public function can_scope_by_id()
    {
        $eventA = factory(Event::class)->create(['slug' => 'hello-world']);
        $eventB = factory(Event::class)->create(['slug' => 'goodbye-world']);

        $event = Event::byIdOrSlug($eventA->id)->first();

        $this->assertSameModel($eventA, $event);
    }

    /** @test **/
    public function can_scope_by_slug()
    {
        $eventA = factory(Event::class)->create(['slug' => 'hello-world']);
        $eventB = factory(Event::class)->create(['slug' => 'goodbye-world']);

        $event = Event::byIdOrSlug($eventB->slug)->first();

        $this->assertSameModel($eventB, $event);
    }
}
